library shake_http_client;

import 'package:http/http.dart' as http;
import 'package:shake_flutter/models/network_request.dart';
import 'package:shake_flutter/shake_flutter.dart';
import 'package:shake_flutter/utils/extensions.dart';

/// Network requests logger for Shake.
class ShakeHttpLogger {
  void logNetworkRequest(
      DateTime startTime, http.Response response, dynamic body) {
    http.Request? request = response.request as http.Request?;
    if (request == null) {
      return;
    }

    NetworkRequest networkRequest = NetworkRequest();
    networkRequest.startTime = startTime;
    networkRequest.endTime = DateTime.now();
    networkRequest.duration = _findDuration(networkRequest);
    networkRequest.url = request.url.toString();
    networkRequest.method = request.method;
    networkRequest.status = response.statusCode.toString();
    networkRequest.requestBody = _removeBinaryData(body?.toString());
    networkRequest.responseBody = _removeBinaryData(response.body);
    request.headers.forEach((header, value) {
      networkRequest.requestHeaders[header] = value;
    });
    response.headers.forEach((header, value) {
      networkRequest.responseHeaders[header] = value;
    });

    Shake.insertNetworkRequest(networkRequest);
  }

  String _removeBinaryData(String? text) {
    if (text == null) return "";
    if (text.isBinary()) return "Binary data";

    return text;
  }

  int _findDuration(NetworkRequest networkRequest) {
    if (networkRequest.startTime != null && networkRequest.endTime != null) {
      return networkRequest.endTime!
          .difference(networkRequest.startTime!)
          .inMilliseconds;
    }
    return 0;
  }
}
